// **********************************************************************
//
// Copyright (c) 2003-2005 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <CallQueue.h>
#include <Ice/Ice.h>

using namespace std;

CallQueue::CallQueue()
    : _destroy(false)
{
}

void
CallQueue::add(const CallPtr& req)
{
    Lock sync(*this);
    if(_exception.get())
    {
        _exception->ice_throw(); // Propagate exceptions to the main thread.
    }
    _req.push_back(req);
    notify();
}

void
CallQueue::destroy()
{
    Lock sync(*this);
    _destroy = true;
    notify();
}

void
CallQueue::run()
{
    while(true)
    {
	CallPtr req;
	{
	    Lock sync(*this);
	    while(!_destroy && _req.empty())
	    {
		wait();
	    }
	    if(_destroy)
	    {
		break;
	    }
	    req = _req.front();
	    _req.pop_front();
	}
	try
	{
	    req->execute(this);
	}
	catch(const Ice::Exception& e)
	{
	    error(e);
	}
    }
}

void
CallQueue::error(const Ice::Exception& e)
{
    Lock sync(*this);
    _exception = auto_ptr<Ice::Exception>(e.ice_clone());
    _destroy = true;
    notify();
}
