// **********************************************************************
//
// Copyright (c) 2003-2010 ZeroC, Inc. All rights reserved.
//
// This copy of Ice is licensed to you under the terms described in the
// ICE_LICENSE file included in this distribution.
//
// **********************************************************************

#include <Ice/Ice.h>
#include <Hello.h>

using namespace std;
using namespace Demo;


void test(HelloPrx& hello);


class CRTEIceCookie: public Ice::LocalObject
{
public:
	CRTEIceCookie(const std::string* m) :
		_m(*m)
	{
	}

	std::string _m;
};

typedef IceUtil::Handle<CRTEIceCookie> CRTEIceCookiePtr;

class CICECallback: public IceUtil::Shared
{
public:

	CICECallback(HelloPrx& hello):hello(hello){}

	void sayhelloICECallback(
			const CRTEIceCookiePtr& cookie)
	{
		//cout<<"async op succeeded "<<cookie->_m<<endl;
		//test(hello);
	}

	void failureICECallback(const Ice::Exception& ex, const CRTEIceCookiePtr& cookie)
	{
		//cout<<"async op failed "<<cookie->_m<<endl;
		//test(hello);
	}
private:
	HelloPrx hello;
};


typedef IceUtil::Handle<CICECallback> CICECallbackPtr;


void test(HelloPrx& hello,string* msg)
{
	CICECallbackPtr callbackObj=new CICECallback(hello);
	Callback_Hello_sayHelloPtr cb = newCallback_Hello_sayHello(callbackObj,&CICECallback::sayhelloICECallback, &CICECallback::failureICECallback);
	CRTEIceCookiePtr cookie = CRTEIceCookiePtr(new CRTEIceCookie(msg));
	hello->begin_sayHello(cb,cookie);
}



int do_test(int argc, char* argv[])
{
    Ice::CommunicatorPtr communicator;
    string msg="test";

    try
    {
        communicator = Ice::initialize(argc, argv);
        HelloPrx hello = HelloPrx::checkedCast(communicator->stringToProxy("hello:tcp -p 10001")->ice_timeout(200));
        //hello->sayHello();

	for(int k=0;k<10;k++)
	{
		for(int i=0;i<5000;i++)
			test(hello,&msg);
		sleep(1);
	}

	

	communicator->shutdown();
	communicator->waitForShutdown();
        communicator->destroy();
    }
    catch(const Ice::Exception& ex)
    {
        cerr << ex << endl;
        try
        {
            if(communicator)
            {
                communicator->destroy();
            }
        }
        catch(const Ice::Exception& ex)
        {
            cerr << ex << endl;
        }
        exit(1);
    }
    return 0;
}

int main(int argc, char* argv[])
{
	for(int i=0;i<40;i++)
	{
		char** argvv=new char*[argc];
		for(int k=0;k<argc;k++)
			argvv[k]=argv[k];

		cout<<"loop: "<<(i+1)<<endl;

		do_test(argc,argvv);	

		delete [] argvv;
	}
}
